# -*- coding: utf-8 -*-
from urllib3 import disable_warnings
from urllib3.exceptions import InsecureRequestWarning
from requests import Session
from bs4 import BeautifulSoup
from ToushinReader.locator import AttributeLocator

disable_warnings(InsecureRequestWarning)


class Fund:
    def __init__(self, isin_code: str):
        url = f"https://toushin-lib.fwg.ne.jp/FdsWeb/FDST030000?isinCd={isin_code}"

        self._soup = BeautifulSoup(
            Session().get(url, verify=False).text, features="html.parser"
        )

    def _parse_element(self, css_selector: str, attr: str = None) -> str:

        res = [elem.get(attr) if attr else elem.text for elem in self._soup.select(css_selector)]

        if len(res) > 0:
            return str(res[0])

    @staticmethod
    def _sanitize(text: str) -> str:
        if text:
            res = (
                text.replace("評価基準日\xa0\xa0", "")
                    .replace("愛称：", "")
                    .replace("運用会社名：", "")
                    .replace("\n", "")
                    .strip()
            )

            return res

    @property
    def as_of_date(self) -> str:
        """
        評価基準日
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.AS_OF_DATE))

    @property
    def name(self) -> str:
        """
        ファンド名
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.FUND_NAME))

    @property
    def nickname(self) -> str:
        """
        愛称
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.FUND_NICKNAME))

    @property
    def investment_manager(self) -> str:
        """
        運用会社名
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.INVESTMENT_MANAGER))

    @property
    def basic_price(self) -> str:
        """
        基準価額
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.BASIC_PRICE))

    @property
    def basic_price_yen_positive_change(self) -> str:
        """
        基準価額前日比変化幅(上昇)
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.BASIC_PRICE_YEN_POSITIVE_CHANGE))

    @property
    def basic_price_pct_positive_change(self) -> str:
        """
        基準価額前日比変化率(上昇)
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.BASIC_PRICE_PCT_POSITIVE_CHANGE))

    @property
    def basic_price_yen_negaitive_change(self) -> str:
        """
        基準価額前日比変化幅(下落)
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.BASIC_PRICE_YEN_NEGATIVE_CHANGE))

    @property
    def basic_price_pct_negative_change(self) -> str:
        """
        基準価額前日比変化率(下落)
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.BASIC_PRICE_PCT_NEGATIVE_CHANGE))

    @property
    def net_asset_amount(self) -> str:
        """
        純資産総額
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.NET_ASSET_AMOUNT))

    @property
    def category(self) -> str:
        """
        商品分類
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.CATEGORY))

    @property
    def index_type(self) -> str:
        """
        インデックス型
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.INDEX_TYPE))

    @property
    def establish_date(self) -> str:
        """
        設定日
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.ESTABLISH_DATE))

    @property
    def closing_date(self) -> str:
        """
        償還日
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.CLOSING_DATE))

    @property
    def date_to_closing(self) -> str:
        """
        償還までの期間
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.DATE_TO_CLOSING))

    @property
    def settlement_freq(self) -> str:
        """
        決算頻度
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.SETTLEMENT_FREQ))

    @property
    def date_of_settlement(self) -> str:
        """
        決算日
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.DATE_OF_SETTLEMENT))

    @property
    def redeem_fee(self) -> str:
        """
        解約手数料
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.REDEEM_FEE))

    @property
    def subscribe_fee(self) -> str:
        """
        購入時手数料
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.SUBSCRIBE_FEE))

    @property
    def redemption_fee(self) -> str:
        """
        信託財産留保額
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.REDEMPTION_FEE))

    @property
    def management_fee(self) -> str:
        """
        運用管理報酬(信託報酬)
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.MANAGEMENT_FEE))

    @property
    def management_fee_for_investment_manager(self) -> str:
        """
        運用管理報酬(信託報酬)運用会社
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.MANAGEMENT_FEE_FOR_INVESTMENT_MANAGER))

    @property
    def management_fee_for_sales(self) -> str:
        """
        運用管理報酬(信託報酬)販売会社
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.MANAGEMENT_FEE_FOR_SALES))

    @property
    def management_fee_for_custody(self) -> str:
        """
        運用管理報酬(信託報酬)信託銀行
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.MANAGEMENT_FEE_FOR_CUSTODY))

    @property
    def historical_data_url(self) -> str:
        """
        基準価額・純資産総額・分配金のCSV データダウンロード
        :return:
        """

        return "https://toushin-lib.fwg.ne.jp" + self._sanitize(self._parse_element(*AttributeLocator.HISTORICAL_DATA_URL))

    @property
    def dividend(self) -> dict:
        """
        最大直近12ヶ月の分配金を取得する
        :return:
        """
        # dividends = []
        dividend_date = []
        dividend_amount = []

        for i in range(12):
            dividend_date_locator = AttributeLocator().get_distribution_date_locator(i)
            dividend_amount_locator = AttributeLocator().get_distribution_amount_locator(i)

            dividend_date.append(self._sanitize(self._parse_element(*dividend_date_locator)))
            dividend_amount.append(self._sanitize(self._parse_element(*dividend_amount_locator)))

        return {
            "date": dividend_date,
            "amount": dividend_amount
        }

    @property
    def dividend_total_of_year(self) -> str:
        """
        年間分配金累計
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.DIVIDEND_TOTAL_OF_YEAR))

    @property
    def fund_code(self) -> str:
        """
        協会ファンドコード
        :return:
        """
        return self._sanitize(self._parse_element(*AttributeLocator.FUND_CODE))

    @property
    def pct_change(self) -> dict:
        """
        騰落率を取得する
        :return:
        """
        period = []
        fund = []
        category = []

        for i in range(5):
            pct_change_period_locator = AttributeLocator().get_pct_change_period_locator(i)
            pct_change_fund_locator = AttributeLocator().get_pct_change_fund_locator(i)
            pct_change_category_locator = AttributeLocator().get_pct_change_category_locator(i)

            period.append(self._sanitize(self._parse_element(*pct_change_period_locator)))
            fund.append(self._sanitize(self._parse_element(*pct_change_fund_locator)))
            category.append(self._sanitize(self._parse_element(*pct_change_category_locator)))

        return {
            "period": period,
            "fund": fund,
            "category": category
        }

    @property
    def risk(self) -> dict:
        """
        騰落率を取得する
        :return:
        """
        period = []
        fund = []
        category = []

        for i in range(5):
            risk_period_locator = AttributeLocator().get_risk_period_locator(i)
            risk_fund_locator = AttributeLocator().get_risk_fund_locator(i)
            risk_category_locator = AttributeLocator().get_risk_category_locator(i)

            period.append(self._sanitize(self._parse_element(*risk_period_locator)))
            fund.append(self._sanitize(self._parse_element(*risk_fund_locator)))
            category.append(self._sanitize(self._parse_element(*risk_category_locator)))

        return {
            "period": period,
            "fund": fund,
            "category": category
        }

    @property
    def sharpe_ratio(self) -> dict:
        """
        騰落率を取得する
        :return:
        """
        period = []
        fund = []
        category = []

        for i in range(5):
            sharpe_ratio_period_locator = AttributeLocator().get_sharpe_ratio_period_locator(i)
            sharpe_ratio_fund_locator = AttributeLocator().get_sharpe_ratio_fund_locator(i)
            sharpe_ratio_category_locator = AttributeLocator().get_sharpe_ratio_category_locator(i)

            period.append(self._sanitize(self._parse_element(*sharpe_ratio_period_locator)))
            fund.append(self._sanitize(self._parse_element(*sharpe_ratio_fund_locator)))
            category.append(self._sanitize(self._parse_element(*sharpe_ratio_category_locator)))

        return {
            "period": period,
            "fund": fund,
            "category": category
        }



# -*- coding: UTF-8 -*-
class AttributeLocator:
    # 評価基準日
    AS_OF_DATE = (
        "body > div.fds-image-top-message.mb-4 > div > div > div:nth-child(1) > div.col-md-3.d-flex.align-items-md-end.flex-md-column.mt-3.mt-md-0 > span",
        None,
    )
    # ファンド名
    FUND_NAME = (
        "body > div.fds-image-top-message.mb-4 > div > div > div:nth-child(1) > div.col-md-9 > h3",
        None,
    )
    # 愛称
    FUND_NICKNAME = (
        "body > div.fds-image-top-message.mb-4 > div > div > div:nth-child(1) > div.col-md-9 > div:nth-child(2)",
        None,
    )
    # 運用会社名
    INVESTMENT_MANAGER = (
        "body > div.fds-image-top-message.mb-4 > div > div > div:nth-child(1) > div.col-md-9 > div:nth-child(3)",
        None,
    )
    # 基準価額
    BASIC_PRICE = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div:nth-child(1) > div.d-flex > span",
        None,
    )
    # 基準価額前日比変化幅(上昇)
    BASIC_PRICE_YEN_POSITIVE_CHANGE = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div:nth-child(2) > div.fds-positive-number-fg > div.d-flex > span",
        None,
    )
    # 基準価額前日比変化率(上昇)
    BASIC_PRICE_PCT_POSITIVE_CHANGE = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div:nth-child(2) > div.fds-positive-number-fg > div.font-weight-normal > span",
        None,
    )
    # 基準価額前日比変化幅(下落)
    BASIC_PRICE_YEN_NEGATIVE_CHANGE = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div:nth-child(2) > div.fds-negative-number-fg > div.d-flex > span",
        None,
    )
    # 基準価額前日比変化率(下落)
    BASIC_PRICE_PCT_NEGATIVE_CHANGE = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div:nth-child(2) > div.fds-negative-number-fg > div.font-weight-normal > span",
        None,
    )
    # 純資産総額
    NET_ASSET_AMOUNT = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div:nth-child(3) > div:nth-child(2)",
        None,
    )
    # 商品分類
    CATEGORY = (
        "body > div.fds-image-top-message.mb-4 > div > div > div.row.no-gutters.font-weight-bold > div.col-6.col-md-3.pl-md-5 > div.fds-font-size-0",
        None,
    )
    # インデックス型
    INDEX_TYPE = ("#tab_content1 > div > table > tr:nth-child(1) > td", None)
    # 設定日
    ESTABLISH_DATE = (
        "#tab_content1 > div > table > tr:nth-child(2) > td",
        None,
    )
    # 償還日
    CLOSING_DATE = ("#tab_content1 > div > table > tr:nth-child(3) > td", None)
    # 償還までの期間
    DATE_TO_CLOSING = (
        "#tab_content1 > div > table > tr:nth-child(4) > td",
        None,
    )
    # 決算頻度
    SETTLEMENT_FREQ = (
        "#tab_content1 > div > table > tr:nth-child(5) > td",
        None,
    )
    # 決算日
    DATE_OF_SETTLEMENT = (
        "#tab_content1 > div > table > tr:nth-child(6) > td",
        None,
    )
    # 解約手数料
    REDEEM_FEE = (
        "#tab_content1 > div > table > tr:nth-child(8) > td",
        None,
    )
    # 購入時手数料
    SUBSCRIBE_FEE = (
        "#tab_content1 > div > div:nth-child(2) > div > div.col-md-7 > table > tbody > tr > td:nth-child(1)",
        None,
    )
    # 信託財産留保額
    REDEMPTION_FEE = (
        "#tab_content1 > div > div:nth-child(2) > div > div.col-md-7 > table > tbody > tr > td:nth-child(2)",
        None,
    )
    # 運用管理報酬(信託報酬)
    MANAGEMENT_FEE = (
        "#tab_content1 > div > div:nth-child(2) > div > div.col-md-7 > table > tbody > tr > td:nth-child(3)",
        None,
    )
    # 運用管理報酬(信託報酬)運用会社
    MANAGEMENT_FEE_FOR_INVESTMENT_MANAGER = (
        "#tab_content1 > div > div:nth-child(2) > div > div.col-md-4 > table > tr:nth-child(1) > td",
        None,
    )
    # 運用管理報酬(信託報酬)販売会社
    MANAGEMENT_FEE_FOR_SALES = (
        "#tab_content1 > div > div:nth-child(2) > div > div.col-md-4 > table > tr:nth-child(2) > td",
        None,
    )
    # 運用管理報酬(信託報酬)信託銀行
    MANAGEMENT_FEE_FOR_CUSTODY = (
        "#tab_content1 > div > div:nth-child(2) > div > div.col-md-4 > table > tr:nth-child(3) > td",
        None,
    )
    # 基準価額・純資産総額・分配金のCSVデータ
    HISTORICAL_DATA_URL = ("#download", "href")

    # 分配金
    DISTRIBUTION = ("#dividendInfo", None)

    # 年間分配金累計
    DIVIDEND_TOTAL_OF_YEAR = ("#dividend1y > tr > td", None)

    # 協会ファンドコード
    FUND_CODE = ("#associFundCd", "value")

    @staticmethod
    def get_distribution_date_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 2
            return (f"#dividendInfo > tr:nth-child({i}) > th", None)

    @staticmethod
    def get_distribution_amount_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 2
            return (f"#dividendInfo > tr:nth-child({i}) > td", None)

    @staticmethod
    def get_pct_change_period_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(1) > div > table > tr:nth-child({i}) > th",
                None)

    @staticmethod
    def get_pct_change_fund_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(1) > div > table > tr:nth-child({i}) > td:nth-child(2) > span",
                None)

    @staticmethod
    def get_pct_change_category_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(1) > div > table > tr:nth-child({i}) > td:nth-child(3) > span",
                None)

    @staticmethod
    def get_risk_period_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(2) > div > table > tr:nth-child({i}) > th",
                None)

    @staticmethod
    def get_risk_fund_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(2) > div > table > tr:nth-child({i}) > td:nth-child(2) > span",
                None)

    @staticmethod
    def get_risk_category_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(2) > div > table > tr:nth-child({i}) > td:nth-child(3) > span",
                None)

    @staticmethod
    def get_sharpe_ratio_period_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(3) > div > table > tr:nth-child({i}) > th",
                None)

    @staticmethod
    def get_sharpe_ratio_fund_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1)> div:nth-child(3) > div > table > tr:nth-child({i}) > td:nth-child(2) > span",
                None)

    @staticmethod
    def get_sharpe_ratio_category_locator(number: int) -> tuple:
        if isinstance(number, int):
            i = number + 3
            return (
                f"#tab_content2 > div > div:nth-child(2) > div:nth-child(1) > div:nth-child(3) > div > table > tr:nth-child({i}) > td:nth-child(3) > span",
                None)
